"use strict";

import { createHighlighterCore } from '@shikijs/core';
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript';
import React, { useEffect, useMemo, useState } from 'react';
import { ScrollView, StyleSheet, Text, View } from 'react-native';
import { trimNewlines } from 'trim-newlines';
import { getRNStylesFromShikiStyle } from './../../utils/style';

// Default theme with basic styles
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
const DEFAULT_THEME = {
  name: 'default',
  type: 'dark',
  colors: {
    'editor.foreground': '#000000',
    'editor.background': '#ffffff'
  },
  settings: [{
    settings: {
      foreground: '#000000',
      background: '#ffffff'
    }
  }]
};
async function createHighlighter(grammar) {
  const highlighter = await createHighlighterCore({
    engine: createJavaScriptRegexEngine(),
    langs: grammar ? [grammar] : undefined
  });
  return highlighter;
}
const SyntaxHighlighter = ({
  code,
  language,
  theme,
  themeLoader,
  grammarDefinition,
  textStyle,
  scrollViewProps,
  containerStyle
}) => {
  const [highlightedNodes, setHighlightedNodes] = useState(null);
  const [error, setError] = useState(null);
  const [loadedTheme, setLoadedTheme] = useState(() => theme ?? DEFAULT_THEME);
  const [isLoading, setIsLoading] = useState(true);
  useEffect(() => {
    const loadResources = async () => {
      setIsLoading(true);
      try {
        // Load theme if themeLoader provided
        let themeToUse = theme ?? DEFAULT_THEME;
        if (themeLoader) {
          try {
            const loadedThemeModule = await themeLoader();
            themeToUse = loadedThemeModule.default;
          } catch (e) {
            console.warn('Failed to load theme:', e);
          }
        }
        setLoadedTheme(themeToUse);

        // Create highlighter with grammar
        const highlighter = await createHighlighter(grammarDefinition);

        // Highlight code
        const tokens = highlighter.codeToHast(code, {
          lang: language,
          theme: themeToUse
        });
        setHighlightedNodes(tokens.children || null);
        setError(null);
      } catch (err) {
        console.error('Syntax highlighting error:', err);
        setError(err instanceof Error ? err.message : 'Unknown error occurred');
        // Fallback to plain text
        setHighlightedNodes([{
          type: 'text',
          value: code
        }]);
      } finally {
        setIsLoading(false);
      }
    };
    loadResources();
  }, [code, language, theme, themeLoader, grammarDefinition]);

  // Create stylesheet from theme, using default theme as fallback
  const stylesheet = useMemo(() => {
    try {
      return getRNStylesFromShikiStyle(loadedTheme);
    } catch (e) {
      console.warn('Failed to create stylesheet:', e);
      return getRNStylesFromShikiStyle(DEFAULT_THEME);
    }
  }, [loadedTheme]);
  const getStylesForNode = node => {
    const classes = node.properties?.className ?? [];
    return classes.map(c => stylesheet[c]).filter(Boolean);
  };
  const renderNode = (nodes, keyPrefix = 'row') => nodes.reduce((acc, node, index) => {
    const keyPrefixWithIndex = `${keyPrefix}_${index}`;
    if (node.children) {
      const styles = StyleSheet.flatten([textStyle, {
        color: stylesheet.shiki?.color
      }, ...getStylesForNode(node)]);
      acc.push(/*#__PURE__*/_jsx(Text, {
        style: styles,
        children: renderNode(node.children, `${keyPrefixWithIndex}_child`)
      }, keyPrefixWithIndex));
    }
    if (node.value) {
      acc.push(trimNewlines(String(node.value)));
    }
    return acc;
  }, []);
  if (isLoading) {
    return /*#__PURE__*/_jsx(View, {
      style: containerStyle,
      children: /*#__PURE__*/_jsx(Text, {
        style: textStyle,
        children: "Loading..."
      })
    });
  }
  if (error) {
    return /*#__PURE__*/_jsxs(View, {
      style: containerStyle,
      children: [/*#__PURE__*/_jsxs(Text, {
        style: [textStyle, {
          color: 'red'
        }],
        children: ["Error rendering code:", error]
      }), /*#__PURE__*/_jsx(Text, {
        style: textStyle,
        children: code
      })]
    });
  }
  return /*#__PURE__*/_jsx(ScrollView, {
    ...scrollViewProps,
    horizontal: true,
    contentContainerStyle: [stylesheet.shiki, scrollViewProps?.contentContainerStyle, containerStyle],
    children: /*#__PURE__*/_jsx(View, {
      onStartShouldSetResponder: () => true,
      children: highlightedNodes && renderNode(highlightedNodes)
    })
  });
};
export default SyntaxHighlighter;
//# sourceMappingURL=SyntaxHighlighter.js.map