import type { HighlighterCore, LanguageRegistration, ThemeRegistrationRaw } from '@shikijs/core'
import type { FunctionComponent, ReactNode } from 'react'
import type { ScrollViewProps, TextStyle, ViewStyle } from 'react-native'
import type { HighlighterStyleSheet } from './../../utils/style'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { useEffect, useMemo, useState } from 'react'
import { ScrollView, StyleSheet, Text, View } from 'react-native'
import { trimNewlines } from 'trim-newlines'
import { getRNStylesFromShikiStyle } from './../../utils/style'

// Default theme with basic styles
const DEFAULT_THEME: ThemeRegistrationRaw = {
  name: 'default',
  type: 'dark',
  colors: {
    'editor.foreground': '#000000',
    'editor.background': '#ffffff',
  },
  settings: [
    {
      settings: {
        foreground: '#000000',
        background: '#ffffff',
      },
    },
  ],
}

interface RendererNode {
  type: string
  value?: string
  children?: RendererNode[]
  properties?: {
    className?: string[]
  }
}

export interface SyntaxHighlighterProps {
  code: string
  language: string
  theme?: ThemeRegistrationRaw
  themeLoader?: () => Promise<{ default: ThemeRegistrationRaw }>
  grammarDefinition?: LanguageRegistration
  textStyle?: TextStyle
  scrollViewProps?: ScrollViewProps
  containerStyle?: ViewStyle
  children?: ReactNode
}

async function createHighlighter(grammar?: LanguageRegistration) {
  const highlighter: HighlighterCore = await createHighlighterCore({
    engine: createJavaScriptRegexEngine(),
    langs: grammar ? [grammar] : undefined,
  })

  return highlighter
}

const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({ code, language, theme, themeLoader, grammarDefinition, textStyle, scrollViewProps, containerStyle }) => {
  const [highlightedNodes, setHighlightedNodes] = useState<RendererNode[] | null>(null)
  const [error, setError] = useState<string | null>(null)
  const [loadedTheme, setLoadedTheme] = useState<ThemeRegistrationRaw>(() => theme ?? DEFAULT_THEME)
  const [isLoading, setIsLoading] = useState(true)

  useEffect(() => {
    const loadResources = async () => {
      setIsLoading(true)
      try {
        // Load theme if themeLoader provided
        let themeToUse = theme ?? DEFAULT_THEME
        if (themeLoader) {
          try {
            const loadedThemeModule = await themeLoader()
            themeToUse = loadedThemeModule.default
          }
          catch (e) {
            console.warn('Failed to load theme:', e)
          }
        }
        setLoadedTheme(themeToUse)

        // Create highlighter with grammar
        const highlighter = await createHighlighter(grammarDefinition)

        // Highlight code
        const tokens = highlighter.codeToHast(code, {
          lang: language,
          theme: themeToUse,
        })

        setHighlightedNodes(tokens.children || null)
        setError(null)
      }
      catch (err) {
        console.error('Syntax highlighting error:', err)
        setError(err instanceof Error ? err.message : 'Unknown error occurred')
        // Fallback to plain text
        setHighlightedNodes([
          {
            type: 'text',
            value: code,
          },
        ])
      }
      finally {
        setIsLoading(false)
      }
    }

    loadResources()
  }, [code, language, theme, themeLoader, grammarDefinition])

  // Create stylesheet from theme, using default theme as fallback
  const stylesheet = useMemo<HighlighterStyleSheet>(() => {
    try {
      return getRNStylesFromShikiStyle(loadedTheme)
    }
    catch (e) {
      console.warn('Failed to create stylesheet:', e)
      return getRNStylesFromShikiStyle(DEFAULT_THEME)
    }
  }, [loadedTheme])

  const getStylesForNode = (node: RendererNode): TextStyle[] => {
    const classes: string[] = node.properties?.className ?? []
    return classes.map((c: string) => stylesheet[c]).filter(Boolean) as TextStyle[]
  }

  const renderNode = (nodes: RendererNode[], keyPrefix = 'row'): ReactNode[] =>
    nodes.reduce<ReactNode[]>((acc, node, index) => {
      const keyPrefixWithIndex = `${keyPrefix}_${index}`
      if (node.children) {
        const styles = StyleSheet.flatten([textStyle, { color: stylesheet.shiki?.color }, ...getStylesForNode(node)])
        acc.push(
          <Text style={styles} key={keyPrefixWithIndex}>
            {renderNode(node.children, `${keyPrefixWithIndex}_child`)}
          </Text>,
        )
      }

      if (node.value) {
        acc.push(trimNewlines(String(node.value)))
      }

      return acc
    }, [])

  if (isLoading) {
    return (
      <View style={containerStyle}>
        <Text style={textStyle}>Loading...</Text>
      </View>
    )
  }

  if (error) {
    return (
      <View style={containerStyle}>
        <Text style={[textStyle, { color: 'red' }]}>
          Error rendering code:
          {error}
        </Text>
        <Text style={textStyle}>{code}</Text>
      </View>
    )
  }

  return (
    <ScrollView {...scrollViewProps} horizontal contentContainerStyle={[stylesheet.shiki, scrollViewProps?.contentContainerStyle, containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{highlightedNodes && renderNode(highlightedNodes)}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
